﻿#############################################################################################################################################
# File:             UserToGroup.ps1                                                                                                         #
# Author:           Jean-Marc Seguin, Chief Architect                                                                                       #
# Publisher:        Embotics Corporation                                                                                                    #
# Copyright:        © 2011 Embotics Corporation. All rights reserved.                                                                       #
# Usage:            To load this module in your PowerShell:                                                                                 #
#                   1. Open PowerShell (or PowerShell ISE).                                                                                 #
#                   2. Run the following commands                                                                                           #
#                      . UserToGroup.ps1	                                                                                                #
#                      Add-UserToLocalAdminGroup -userName <username> [-domainName <domain> -serverName <server_fqdn_or_ip>]                #
#                      Add-UserToGroup -groupName <group> -userName <username> [-domainName <domain> -serverName <server_fqdn_or_ip>]       #
#                      Remove-UserToLocalAdminGroup -userName <username> [-domainName <domain> -serverName <server_fqdn_or_ip>]             #
#                      Remove-UserToGroup -groupName <group> -userName <username> [-domainName <domain> -serverName <server_fqdn_or_ip>]    #
#############################################################################################################################################	

function Add-UserToLocalAdminGroup {
<#
.Synopsis
	Add a user account to the local administrators group on a remote server (or localhost)

.Description
	The Add-UserToLocalAdminGroup function takes a username with optional domain parameter and adds it to tje local administrators group on either the specified remote server or localhost if no server specified

.Parameter $userName
	The username to add
.Parameter $domainName
	The domain to use. Optional: If no domain specified, the domain of the local system will be used (ie presumed to be a local account)
.Parameter $serverName
	The server to connect to. Optional: If no server specified, localhost will be used
	
.Example
	C:\PS> .\Add-UserToLocalAdminGroup.ps1 -userName jsmith
	Adds the user 'jsmith' to the local administrators group on this computer

.Example
	C:\PS> .\Add-UserToLocalAdminGroup.ps1 -userName jsmith -server acm-001.acme.com
	Adds the user 'jsmith' to the local administrators group on the remote computer called 'acm-001.acme.com'

.Example
	C:\PS> .\Add-UserToLocalAdminGroup.ps1 -userName jsmith -domainName ACME -server acm-002.acme.com
	Adds the user 'ACME\jsmith' to the local administrators group on the server called 'acm-002.acme.com'

.Example
	C:\PS> .\Add-UserToLocalAdminGroup.ps1 -userName jsmith -domainName ACME
	Adds the user 'ACME\jsmith' to the local administrators group on this computer

#>

#[CmdletBinding()]
	param(
		[Parameter(Position=0, Mandatory=$true)]
		[alias("user", "account")]
		$userName = $(Throw "Please provide the username you want to add to the local Administrators group"), 
	
		[Parameter(Position=1, Mandatory=$false)]
		[alias("domain")]
		$domainName,
	
		[Parameter(Position=2, Mandatory=$false)]
		[alias("server")]
		$serverName = '.'
	)


	Process
	{  
		Add-UserToGroup -groupName administrators -userName $userName -domainName $domainName -serverName $serverName
		
	}# Process
}

Function Add-UserToGroup {
<#
.Synopsis
	Add a user account to the specified local group on a remote server (or localhost)

.Description
	The Add-UserToGroup function takes a username with optional domain parameter and adds it to the specified local group on either the specified 
	remote server or localhost if no server specified

.Parameter $groupName
	The group to add the user to. 
.Parameter $userName
	The username to add
.Parameter $domainName
	The domain to use. Optional: If no domain specified, the domain of the local system will be used (ie presumed to be a local account)
.Parameter $serverName
	The server to connect to. Optional: If no server specified, localhost will be used
	
.Example
	C:\PS> .\Add-UserToGroup.ps1 -groupName Administrators -userName jsmith
	Adds the user 'jsmith' to the local administrators group on this computer

.Example
	C:\PS> .\Add-UserToLocalAdminGroup.ps1 -groupName Administrators -userName jsmith -server acm-001.acme.com
	Adds the user 'jsmith' to the local administrators group on the remote computer called 'acm-001.acme.com'

.Example
	C:\PS> .\Add-UserToLocalAdminGroup.ps1 -groupName Administrators -userName jsmith -domainName ACME -server acm-002.acme.com
	Adds the user 'ACME\jsmith' to the local administrators group on the server called 'acm-002.acme.com'

.Example
	C:\PS> .\Add-UserToLocalAdminGroup.ps1 -groupName Administrators -userName jsmith -domainName ACME
	Adds the user 'ACME\jsmith' to the local administrators group on this computer

#>

#[CmdletBinding()]
	param(
		[Parameter(Position=0, Mandatory=$true)]
		[alias("group", "memberof")]
		$groupName = $(Throw "Please provide the group name you want to add the user to"), 

		[Parameter(Position=1, Mandatory=$true)]
		[alias("user", "account")]
		$userName = $(Throw "Please provide the username you want to add to the local Administrators group"), 
	
		[Parameter(Position=2, Mandatory=$false)]
		[alias("domain")]
		$domainName,
	
		[Parameter(Position=3, Mandatory=$false)]
		[alias("server")]
		$serverName = '.'
	)


	Process
	{  
		#if localhost, set to correct name
	    if($serverName -eq '.'){$serverName = (get-WmiObject win32_computersystem).Name}    
    	$serverName = $serverName.ToUpper()
		
		if(!$domainName){
			if($env:USERDNSDOMAIN){
				$domainName = $env:USERDNSDOMAIN
			}
			else{
				$domainName = $serverName
			}
		}

		Write-Output "groupName: $groupName"
		Write-Output "userName: $userName"
		Write-Output "domainName: $domainName"
		Write-Output "serverName: $serverName"
		
		$adsi = [ADSI]"WinNT://$serverName/$groupName,group"
        $adsi.add("WinNT://$domainName/$userName,group")
		Write-Output "$userName added to $groupName on $serverName"
	}# Process
}


function Remove-UserFromLocalAdminGroup {
<#
.Synopsis
	Remove a user account to the local administrators group on a remote server (or localhost)

.Description
	The Remove-UserToLocalAdminGroup function takes a username with optional domain parameter and removes it from the local administrators group on either the specified remote server or localhost if no server specified

.Parameter $userName
	The username to remove
.Parameter $domainName
	The domain to use. Optional: If no domain specified, the domain of the local system will be used (ie presumed to be a local account)
.Parameter $serverName
	The server to connect to. Optional: If no server specified, localhost will be used
	
.Example
	C:\PS> .\Remove-UserToLocalAdminGroup.ps1 -userName jsmith
	Removes the user 'jsmith' from the local administrators group on this computer

.Example
	C:\PS> .\Remove-UserToLocalAdminGroup.ps1 -userName jsmith -server acm-001.acme.com
	Removes the user 'jsmith' from the local administrators group on the remote computer called 'acm-001.acme.com'

.Example
	C:\PS> .\Remove-UserToLocalAdminGroup.ps1 -userName jsmith -domainName ACME -server acm-002.acme.com
	Removes the user 'ACME\jsmith' from the local administrators group on the server called 'acm-002.acme.com'

.Example
	C:\PS> .\Remove-UserToLocalAdminGroup.ps1 -userName jsmith -domainName ACME
	Removes the user 'ACME\jsmith' from the local administrators group on this computer

#>

#[CmdletBinding()]
	param(
		[Parameter(Position=0, Mandatory=$true)]
		[alias("user", "account")]
		$userName = $(Throw "Please provide the username you want to remove to the local Administrators group"), 
	
		[Parameter(Position=1, Mandatory=$false)]
		[alias("domain")]
		$domainName,
	
		[Parameter(Position=2, Mandatory=$false)]
		[alias("server")]
		$serverName = '.'
	)


	Process
	{  
		Remove-UserToGroup -groupName administrators -userName $userName -domainName $domainName -serverName $serverName
		
	}# Process
}

Function Remove-UserToGroup {
<#
.Synopsis
	Remove a user account to the specified local group on a remote server (or localhost)

.Description
	The Remove-UserToGroup function takes a username with optional domain parameter and removes it to the specified local group on either the specified 
	remote server or localhost if no server specified

.Parameter $groupName
	The group to remove the user from. 
.Parameter $userName
	The username to remove
.Parameter $domainName
	The domain to use. Optional: If no domain specified, the domain of the local system will be used (ie presumed to be a local account)
.Parameter $serverName
	The server to connect to. Optional: If no server specified, localhost will be used
	
.Example
	C:\PS> .\Remove-UserToGroup.ps1 -groupName Administrators -userName jsmith
	Removes the user 'jsmith' from the local administrators group on this computer

.Example
	C:\PS> .\Remove-UserToGroup.ps1 -groupName Administrators -userName jsmith -server acm-001.acme.com
	Removes the user 'jsmith' from the local administrators group on the remote computer called 'acm-001.acme.com'

.Example
	C:\PS> .\Remove-UserToGroup.ps1 -groupName Administrators -userName jsmith -domainName ACME -server acm-002.acme.com
	Removes the user 'ACME\jsmith' from the local administrators group on the server called 'acm-002.acme.com'

.Example
	C:\PS> .\Remove-UserToGroup.ps1 -groupName Administrators -userName jsmith -domainName ACME
	Removes the user 'ACME\jsmith' from the local administrators group on this computer

#>

#[CmdletBinding()]
	param(
		[Parameter(Position=0, Mandatory=$true)]
		[alias("group", "memberof")]
		$groupName = $(Throw "Please provide the group name you want to add the user to"), 

		[Parameter(Position=1, Mandatory=$true)]
		[alias("user", "account")]
		$userName = $(Throw "Please provide the username you want to add to the local Administrators group"), 
	
		[Parameter(Position=2, Mandatory=$false)]
		[alias("domain")]
		$domainName,
	
		[Parameter(Position=3, Mandatory=$false)]
		[alias("server")]
		$serverName = '.'
	)


	Process
	{  
		#if localhost, set to correct name
	    if($serverName -eq '.'){$serverName = (get-WmiObject win32_computersystem).Name}    
    	$serverName = $serverName.ToUpper()
		
		if(!$domainName){
			if($env:USERDNSDOMAIN){
				$domainName = $env:USERDNSDOMAIN
			}
			else{
				$domainName = $serverName
			}
		}

		#write-host "groupName:" $groupName
		#write-host "userName:" $userName
		#Write-Host "domainName:" $domainName
		#Write-Host "serverName:" $serverName
		
		$adsi = [ADSI]"WinNT://$serverName/$groupName,group"
        $adsi.remove("WinNT://$domainName/$userName,group")
		Write-Output "$userName removed from $groupName on $serverName"
	}# Process
}