#######################################################################################################################
# File:             Thin2Thick.ps1                                                                                    #
# Author:           Jean-Marc Seguin, Chief Architect                                                                 #
# Publisher:        Embotics Corporation                                                                              #
# Copyright:         2011 Embotics Corporation. All rights reserved.                                                 #
# Usage:            To load this module in your PowerCLI (or PowerShell with VMware SDK Installed):                   #
#                   1. Open PowerCLI (or PowerShell).                                                                 #
#                   2. Run the following commands                                                                     #
#                      . <install_dir>/Thin2Thick.ps1 -vcenter <server_fqdn_or_ip> -vmname <vmid>                     #
#                            -user <vcenter admin> -password <password>                                               #
#######################################################################################################################
<#

.SYNOPSIS

Covert all thin disks of an identified VM to thick

.EXAMPLE

. ./Thin2Thick.ps1 -vcenter "fqdn or ip" -vmname "Virtual Machine 3" -user administrator -pass passw0rd

.EXAMPLE
If VM is running, power it off first (it will not restart it)
. ./Thin2Thick.ps1 -vcenter "fqdn or ip" -vmname "Virtual Machine 3" -user administrator -pass passw0rd -Force
#>
param
(
	[parameter(Mandatory=$true,
               ValueFromPipeline=$false,
			   HelpMessage="Enter the vCenter server FQDN or IP address")]
	[string] $vcenter = $(Throw "Please specify the vCenter server FQDN or IP address"), 
	[parameter(Mandatory=$true,
               ValueFromPipeline=$false,
			   HelpMessage="Enter the Name of the VM to have disks converted")]
	[string] $vmname = $(Throw "Please specify the Name of the VM to have disks converted"), 
	[parameter(Mandatory=$true,
               ValueFromPipeline=$false,
			   HelpMessage="Enter the vCenter admin username")]
	[string] $user = $(Throw "Please provide vCenter admin username"), 
	[parameter(Mandatory=$true,
               ValueFromPipeline=$false,
			   HelpMessage="Enter the vCenter admin password")]
	[string] $pass = $(Throw "Please provide vCenter admin password"),
	[parameter(Mandatory=$false,
               ValueFromPipeline=$false,
			   HelpMessage="Enter '-Force' to power off the VM before inflating the disks")]
	[switch] $Force, #Force power off if not already off
	[parameter(Mandatory=$false,
               ValueFromPipeline=$false,
			   HelpMessage="Enter 'Restart' to power on the VM after inflating the disks (Only if -Force was selected)")]
	[switch] $Restart #Restart VM if Force power off was chosen
)
Set-StrictMode -Version 2
$ErrorActionPreference = "Stop"
$DEBUG=$true
#$DEBUGFILE="$HOME\Downloads\Thin2Thick-$(get-date -format MMddyyHHmmss).log" #Put logs in Service Account directory
$DEBUGFILE="c:\Logs\Thin2Thick-$(get-date -format MMddyyHHmmss).log" #Put logs in a common directory
$retval=0
if($DEBUG){Write-Output "Start Thin2Thick.ps1" | Out-File $DEBUGFILE -Append}

if($DEBUG){Write-Output "Loading PowerCLI(VimAutomation.Core) Libraries" | Out-File $DEBUGFILE -Append}
add-pssnapin VMware.VimAutomation.Core -ErrorAction SilentlyContinue #Make sure PowerCLI is loaded

if($DEBUG){Write-Output "Params:" | Out-File $DEBUGFILE -Append}
if($DEBUG){Write-Output "vCenter: $vcenter" | Out-File $DEBUGFILE -Append}
if($DEBUG){Write-Output "VM Name: $vmname" | Out-File $DEBUGFILE -Append}
if($DEBUG){Write-Output "User: $user" | Out-File $DEBUGFILE -Append}
if($DEBUG){Write-Output "Password: $pass" | Out-File $DEBUGFILE -Append}
if($DEBUG){Write-Output "Force: $Force" | Out-File $DEBUGFILE -Append}
if($DEBUG){Write-Output "Restart: $Restart" | Out-File $DEBUGFILE -Append}

#Make sure PowerShell as PowerCLI v5 libraries installed	
if($DEBUG){Write-Output "Make sure we have correct PowerCLI libraries" | Out-File $DEBUGFILE -Append}
if (Get-Command Get-PowerCliVersion -ErrorAction SilentlyContinue) {
	if((Get-PowerCliVersion).Major -lt 5) {
		if($DEBUG){Write-Output "Need PowerCLI 5 or higher for this script to work." | Out-File $DEBUGFILE -Append}
		$retval=1
    }
}
else{
	if($DEBUG){Write-Output "Need PowerCLI 5 or higher for this script to work." | Out-File $DEBUGFILE -Append}
	$retval= 1
}
if($retval -gt 0){return $retval}	
	
try{
	#...to log into vCenter
	if($DEBUG){Write-Output "Connect to vCenter." | Out-File $DEBUGFILE -Append}
	Connect-VIServer -Server $vcenter -User $user -Password $pass  -ErrorAction SilentlyContinue | Out-Null
}
catch{
	if($DEBUG){Write-Output "Unable to connect to vCenter \"$vcenter\" as $user." | Out-File $DEBUGFILE -Append}
	$retval= 2
}
if($retval -gt 0){return $retval}	
	
try{
	#...to locate the disks
		#Write-Output "Locate Disks"
	$vm=Get-VM -Name $vmname
	$disks=Get-HardDisk -VM $vm
}
catch{
	if($DEBUG){Write-Output "Unable to locate disks for VM $vmname on $vcenter" | Out-File $DEBUGFILE -Append}
	$retval= 3
}
if($retval -gt 0){return $retval}	

try{
	#...to locate the disks
	if($DEBUG){Write-Output "Is VM Powered Off?" | Out-File $DEBUGFILE -Append}
	if($vm.PowerState -ne "PoweredOff"){
		if($Force){
			#Note: when bug fixed around VMTools Version, this script will do polite guest shutdown instead
			if($DEBUG){Write-Output "No...shutting down VM" | Out-File $DEBUGFILE -Append}
			Stop-VM -VM $vm -Confirm:$false | Wait-Job -ErrorAction SilentlyContinue | Out-Null
		}
		else{
			if($DEBUG){Write-Output "No...BUT not allowed to shut down VM" | Out-File $DEBUGFILE -Append}
			$retval= 4
		}
	}	
	else{
		if($DEBUG){Write-Output "Yes...So good to go" | Out-File $DEBUGFILE -Append}
	}
}
catch{
	if($DEBUG){Write-Output "Unable to power off VM $vmname on $vcenter" | Out-File $DEBUGFILE -Append}
	$retval= 5
}	
if($retval -gt 0){return $retval}	

try{
	#...to inflate the disk
	if($DEBUG){Write-Output "Inflate Thins Only" | Out-File $DEBUGFILE -Append}
	foreach ($disk in $disks) {
		if($disk.StorageFormat -eq "thin"){
			if($DEBUG){Write-Output "$disk.Name is thin. Attempting to inflate" | Out-File $DEBUGFILE -Append}
			$result=Set-HardDisk -HardDisk $disk -Inflate -Confirm:$false
			if($DEBUG){Write-Output "Done Inflating" | Out-File $DEBUGFILE -Append}
		}
		else{
			if($DEBUG){Write-Output "$disk.Name is NOT thin. Skipping" | Out-File $DEBUGFILE -Append}
		}
	}
	if($DEBUG){Write-Output "Done Inflating Thins Only" | Out-File $DEBUGFILE -Append}
}
catch{
	if($DEBUG){Write-Output "Unable to inflate $disk.Name for VM $vmname on $vcenter" | Out-File $DEBUGFILE -Append}
	$retval= 6
}
if($retval -gt 0){return $retval}	

try{
	#...Power on VM if Necessary
	if($vm.PowerState -eq "PoweredOff" -and $Force -and $Restart){
		if($DEBUG){Write-Output "Did I power off the VM?" | Out-File $DEBUGFILE -Append}
		if($DEBUG){Write-Output "Yes...starting up VM" | Out-File $DEBUGFILE -Append}
		Start-VM -VM $vm -Confirm:$false | Wait-Job -ErrorAction SilentlyContinue | Out-Null
	}
}
catch{
	if($DEBUG){Write-Output "Unable to power on VM $vmname on $vcenter" | Out-File $DEBUGFILE -Append}
	$retval= 7
}	
if($retval -gt 0){return $retval}	

try{
	Disconnect-VIServer -Confirm:$false -ErrorAction SilentlyContinue
}
catch{
	if($DEBUG){Write-Output "Unable to disconnect to vCenter $vcenter as $user" | Out-File $DEBUGFILE -Append}
	$retval= 8
}

if($DEBUG){Write-Output "Return code: $retval" | Out-File $DEBUGFILE -Append}
if($DEBUG){Write-Output "End Thin2Thick.ps1" | Out-File $DEBUGFILE -Append}
return $retval


