#######################################################################################################################
# File:             Check-DNSEntry.ps1                                                                                #
# Author:           Jean-Marc Seguin, Chief Architect                                                                 #
# Publisher:        Embotics Corporation                                                                              #
# Copyright:         2012 Embotics Corporation. All rights reserved.                                                 #
# Usage:            To load this module in your PowerShell:                                                           #
#                   1. Open PowerShell (or PowerShell ISE).                                                           #
#                   2. Run the following commands                                                                     #
#                      ./Check-DNSEntry.ps1                                                                           #
#                      Check-DNSByName -server <FQDN or short name>                                                   #
#                      Check-DNSByAddress -server <ip address>                                                         #
#######################################################################################################################

Set-StrictMode -Version 2

<#
.SYNOPSIS

Function to verify existence of a DNS entry for supplied server FQDN or short name

.DESCRIPTION
This utility takes parameters from V-Commander, connects to DNS Server to look up server entry

.PARAMETER server
FQDN or shortname of server to lookup

.EXAMPLE
C:\PS>Check-DNSByName -server host.example.com
10.10.10.27

Description
-----------
This will lookup the server "host" using its fully qualified domain name and return the list of IP Addresses. This is usefull for DNS servers that can resolve multiple domains.

.EXAMPLE
C:\PS>Check-DNSByName -server host
10.10.10.27

Description
-----------
This will lookup the server "host" using its short name. The DNS server will presume default domain for its lookups.

#>
function Check-DNSByName {
    param
    (
		[parameter(Mandatory=$true,
				   ValueFromPipeline=$true,
				   HelpMessage="Please provide a FQDN or shortname of server to check for")]
		[string] $server = $null
    )

	begin {
    }
    process {
		[string]$result = 'no results found' #$null
        if ($server -and $_) {
            Throw 'Please use either pipeline or input parameter to pass FQDN or short name'
            break
        } elseif ($server) {
            $result=([System.Net.Dns]::GetHostByName($server)).AddressList
        } elseif ($_) {
            $result=([System.Net.Dns]::GetHostByName($_)).AddressList
        } else {
            $server = Read-Host "Please provide the FQDN or short name of server"
            $result=([System.Net.Dns]::GetHostByName($server)).AddressList
        }
	    return $result
   	}
    end {
    }
}

<#
.SYNOPSIS

Function to verify existence of a DNS entry for supplied server IP Addresss

.DESCRIPTION
This utility takes parameters from V-Commander, connects to DNS Server to look up server entry

.PARAMETER server
IP Address of server to lookup

.EXAMPLE
C:\PS>Check-DNSByAddress -server 10.10.10.27
server-name

Description
-----------
This will lookup the server by the provided IP Address and return its name

#>
function Check-DNSByAddress {
    param
    (
		[parameter(Mandatory=$true,
				   ValueFromPipeline=$true,
				   HelpMessage="Please provide the IP Address of server to check for")]
		[string] $server = $null
    )
	begin {
    }
    process {
		[string]$result = 'no results found' #$null
        if ($server -and $_) {
            Throw 'Please use either pipeline or input parameter to pass IP Address'
            break
        } elseif ($server) {
            $result=([System.Net.Dns]::GetHostbyAddress($server)).HostName
        } elseif ($_) {
            $result=([System.Net.Dns]::GetHostbyAddress($_)).HostName
        } else {
            $server = Read-Host "Please provide the IP Address"
            $result=([System.Net.Dns]::GetHostbyAddress($server)).HostName
        }
		return $result
    }
    end {
    }
}

#region testcast
$debug=$true
if($debug){Check-DNSByName -server www.google.com}

if($debug){Check-DNSByAddress -server 10.10.10.10}
#endregion
